Package: xmlwriter
Title: Fast and Elegant XML Generation
Version: 0.1.1
Authors@R: 
    person("Edwin", "de Jonge", , "edwindjonge@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6580-4718"))
Description: Provides a fast and elegant interface for generating XML
 fragments and documents. It can be used in companion with R packages 'XML'
 or 'xml2' to generate XML documents. The fast XML generation is implemented using the
 'Rcpp' package. 
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
LinkingTo: Rcpp
Imports: Rcpp
Suggests: xml2, tinytest
URL: https://edwindj.github.io/xmlwriter/
NeedsCompilation: yes
Packaged: 2024-09-25 07:05:36 UTC; edwin
Author: Edwin de Jonge [aut, cre] (<https://orcid.org/0000-0002-6580-4718>)
Maintainer: Edwin de Jonge <edwindjonge@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-25 10:20:03 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:11:07 UTC; unix
Archs: xmlwriter.so.dSYM
