# Generated by roxygen2: do not edit by hand

S3method("+",xml_fragment)
S3method("+",xmlbuilder)
S3method("/",xml_fragment)
S3method("/",xmlbuilder)
S3method(">",xml_fragment)
S3method("^",xmlbuilder)
S3method(as.character,xml_doc)
S3method(as.character,xml_fragment)
S3method(as.character,xmlbuilder)
S3method(c,xml_fragment)
S3method(print,xml_fragment)
S3method(print,xmlbuilder)
S3method(xml2::as_list,xml_fragment)
S3method(xml2::as_xml_document,xml_fragment)
S3method(xml2::as_xml_document,xmlbuilder)
S3method(xml2::write_xml,xml_fragment)
S3method(xml2::write_xml,xmlbuilder)
export(add_child_fragment)
export(as_frag)
export(as_xml_nodeset)
export(data_frag)
export(elem)
export(frag)
export(list_as_xml_document)
export(list_as_xml_string)
export(read_fragment)
export(tag)
export(xml_doc)
export(xml_fragment)
export(xmlbuilder)
importFrom(Rcpp,sourceCpp)
useDynLib(xmlwriter, .registration = TRUE)
